/**
* @file example_ffc_ble.c
* @author www.tuya.com
* @brief example_ffc_ble module is used to 
* @version 0.1
* @date 2022-05-20
*
* @copyright Copyright (c) tuya.inc 2022
*
*/

#include "tuya_cloud_types.h"

#include "tal_log.h"
#include "tuya_bt.h"

/***********************************************************
*************************micro define***********************
***********************************************************/
#define BLE_KEY_TYPE_CMD                0x01   //健值
#define BLE_BIND_CMD                    0x02   //绑定
#define BLE_UNBIND_CMD                  0x03   //解绑
#define BLE_SWITCH_CMD                  0x04   //开关
#define BLE_FAVORITES_CMD               0x05   //喜好收藏
#define BLE_SCHEDULED_CLOSE_CMD         0x06   //倒计时
#define BLE_GROUP_CHECK_CMD             0x07   //一键群组查询

#define BLE_LIGHT_SWITCH_CMD            0x08   //照明开关
#define BLE_LIGHT_BRIGHT_STEP_CMD       0x09   //照明步进调亮度
#define BLE_LIGHT_BRIGHT_STEPLESS_CMD   0x0a   //照明无极调亮度
#define BLE_LIGHT_TEMP_STEP_CMD         0x0b   //照明步进调色温
#define BLE_LIGHT_TEMP_STEPLESS_CMD     0x0c   //照明无极调色温
#define BLE_LIGHT_SEVEN_COLOR_CMD       0x0d   //七彩调节
#define BLE_LIGHT_COLOE_H_STEP_CMD      0x0e   //H 值调节
#define BLE_LIGHT_COLOE_H_DIMMING_CMD   0x0f   //H 值无极调节
#define BLE_LIGHT_COLOE_S_STEP_CMD      0x10   //S 值调节
#define BLE_LIGHT_COLOE_S_DIMMING_CMD   0x11   //S 值无极调节
#define BLE_LIGHT_COLOE_V_STEP_CMD      0x12   //V 值调节
#define BLE_LIGHT_COLOE_V_DIMMING_CMD   0x13   //S 值无极调节
#define BLE_LIGHT_COLOE_HSV_CMD         0x14   //HSV 值调节
#define BLE_LIGHT_SCENE_CHANGE_CMD      0x15   //场景调节
#define BLE_LIGHT_MODE_SET_CMD          0x16   //模式设置
#define BLE_SPECIAL_MODE               0x17   //夜灯模式

#define BLE_RGBY_CONTROL_CMD            0xff   //定制rgby方向调节颜色

/***********************************************************
***********************typedef define***********************
***********************************************************/


/***********************************************************
***********************variable define**********************
***********************************************************/

/**
* @brief 
*
* @param[in] data: data
* @param[in] len: data len
* @param[in] type: type
* @param[in] mac: device mac
* @return none
*/
STATIC VOID ble_receive_callback(UCHAR_T *data, UCHAR_T len,  UCHAR_T type, UCHAR_T* mac)
{
    INT_T i = 0;

    if(type == 0xff){
        TAL_PR_DEBUG("ble remote(old ver): data len: %d", len);
        for ( i = 0; i < len; i++) {
            TAL_PR_DEBUG("data[%d] : %d", i, data[i]);        
        }
    }else if(type == 0x16){
        TAL_PR_DEBUG("ble remote(new ver): data len: %d", len);
        UCHAR_T *data_cmd = (data + 2);

        //解析控制数据
        switch(data_cmd[2]){
            case BLE_BIND_CMD:
                //do binding operation
                TAL_PR_NOTICE("=====bind success!!!=====");
                break;

            case BLE_UNBIND_CMD:
                //do unbinding operation
                TAL_PR_NOTICE("====unbind success!!!====");
                break;

            default:
                //do ctrl operation
                TAL_PR_DEBUG("====please do ctrl operation!!!==== cmd: [%02x], cmd len: [%02d]", data_cmd[2], len - 2);
                // do_handle(&(data_cmd[2]),len-2); 
                break;
        }
    }

    return;
}

/**
* @brief ble scanf adv bind check callback
*
* @param[in] type: bind type for bluetooth remote controller
* @param[in] data: data
* @param[in] len: data len
* @return OPRT_OK
*/
STATIC OPERATE_RET ble_scan_adv_bind_check_callback(TUYA_BLE_BIND_TYPE type, UCHAR_T *data, UCHAR_T len)
{
    INT_T i = 0;
    
    TAL_PR_DEBUG("----------------scan adv bind check cb-----------------");
    TAL_PR_DEBUG("ble bind type : %d, data len: %d", type, len);
    for(i=0;i<len;i++) {
        TAL_PR_DEBUG("data[%d] : %d", i, data[i]);
    }

    return OPRT_OK;
}

/**
* @brief ble scan adv bind notify callback
*
* @param[in] type: bind type for bluetooth remote controller
* @return none
*/
STATIC VOID_T ble_adv_scan_bind_notify_callback(TUYA_BLE_BIND_TYPE type, int rslt)
{
    TAL_PR_DEBUG("------------------ble scan adv bind notify cb----------------");
    TAL_PR_DEBUG("ble bind type : %s", (type == TUYA_BLE_OP_BIND) ? "bind" : "unbind");
    TAL_PR_DEBUG("result : %s", (rslt == 0) ? "success" : "failed");

    return ;
}

/**
* @brief  ffc task
*
* @param[in] param:Task parameters
* @return none
*/
VOID ble_remote_init(VOID)
{
    OPERATE_RET rt = OPRT_OK;
    TUYA_BLE_SCAN_ADV_HANDLE_CBS ble_scan_cfg = {0};
    ble_scan_cfg.bind_check = ble_scan_adv_bind_check_callback;
    ble_scan_cfg.bind_notify = ble_adv_scan_bind_notify_callback;

    /* Register callback function for advertisement scanning data processing */
    TUYA_CALL_ERR_GOTO(tuya_ble_reg_app_scan_adv_cb(ble_receive_callback), __EXIT);

    /* Register callback function for advertisement scanning data processing */
    TUYA_CALL_ERR_GOTO(tuya_ble_reg_app_scan_adv_handle_cbs(&ble_scan_cfg), __EXIT);

    /* set bind window (s) */
    tuya_ble_set_bind_window(30);

__EXIT:
    return;
}
